     ;standard routines needed for pretty much all games

     ;NMI and IRQ handlers
NMI
     ;VISIBLEOVER is 255 while the screen is drawn, and 0 right after the visible screen is done.
     pha
     lda visibleover
     eor #255
     sta visibleover
     pla
IRQ
     RTI


clearscreen
     ldx #(WZONECOUNT-1)
     lda #0
clearscreenloop
     sta dlend,x
     dex
     bpl clearscreenloop
     lda #0
     sta valbufend ; clear the bcd value buffer
     sta valbufendsave 
     rts

restorescreen

     ldx #(WZONECOUNT-1)

     lda #0
restorescreenloop
     lda dlendsave,x
     sta dlend,x
     dex
     bpl restorescreenloop
     lda valbufendsave
     sta valbufend
     rts

savescreen
     ldx #(WZONECOUNT-1)
savescreenloop
     lda dlend,x
     sta dlendsave,x
     dex
     bpl savescreenloop
     lda valbufend
     sta valbufendsave
     rts

drawscreen
     jsr servicesfxchannels 
     jsr checkjoybuttons

     inc framecounter
     lda framecounter
     and #63
     bne skipcountdownseconds
     lda countdownseconds
     beq skipcountdownseconds
     dec countdownseconds

skipcountdownseconds
     lda #0
     sta temp1 ; not B&W if we're here...

drawscreenwait
     ifconst debugcolor
         lda #$00
         sta BACKGRND
     endif
     lda visibleover
     bne drawscreenwait ; make sure the visible screen isn't being drawn

     ifconst AVOXVOICE
         jsr processavoxvoice
     endif

     ;restore some registers in case the user changed them mid-screen...
     lda sCTRL
     ora temp1
     sta CTRL
     lda sCHARBASE
     sta CHARBASE

     ;add DL end entry on each DL
     ldx #(WZONECOUNT-1)

dlendloop
     lda DLPOINTL,x
     sta dlpnt
     lda DLPOINTH,x
     sta dlpnt+1
     ldy dlend,x
     lda #$00
     iny
     sta (dlpnt),y
     dex
     bpl dlendloop

     ifnconst pauseroutineoff
         ; check to see if pause was pressed and released
pauseroutine
         lda #8
         bit SWCHB
         beq pausepressed
         ;pause isn't pressed
         lda #0
         sta pausebuttonflag ; clear pause hold state in case its set

         ;check if we're in an already paused state
         lda pausestate
         beq leavepauseroutine ; nope, leave

         cmp #1 ; last frame was the start of pausing
         beq enterpausestate2 ; move from state 1 to 2

         cmp #2
         beq carryonpausing

         ;pausestate must be >2, which means we're ending an unpause 
         lda #0
         sta pausebuttonflag 
         sta pausestate 
         lda sCTRL
         sta CTRL
         jmp leavepauseroutine

pausepressed
         ;pause is pressed
         lda pausebuttonflag
         cmp #$ff
         beq carryonpausing

         ;its a new press, increment the state
         inc pausestate

         ;silence volume at the start and end of pausing
         lda #0 
         sta AUDV0
         sta AUDV1

         ifconst pokeysupport
             ldy #7
pausesilencepokeyaudioloop
             sta (pokeybase),y
             dey
             bpl pausesilencepokeyaudioloop
         endif

         lda #$ff
         sta pausebuttonflag
         bne carryonpausing

enterpausestate2
         lda #2
         sta pausestate
         bne carryonpausing
leavepauseroutine
         jmp visiblescreenstarted

carryonpausing
         ifconst .pause
             jsr .pause
         endif
         lda #%10000000 ; turn off colorburst during pause...
         sta temp1
         jmp drawscreenwait ; going to drawscreenwait skips servicing the sfx routine, so sfx pause too

     endif

     ; Make sure the visible screen has *started* before we exit. That way we can rely on drawscreen
     ; delaying a full frame, but still allowing time for basic calculations.
visiblescreenstarted
     ifconst debugcolor
         lda #$41
         sta BACKGRND
     endif
visiblescreenstartedwait
     lda visibleover
     beq visiblescreenstartedwait
     rts

     ifconst AVOXVOICE
processavoxvoice
         lda avoxenable
         bne avoxfixport
         SPKOUT temp1
         rts
avoxfixport
         lda #0 ; restore the port to all bits as inputs...
         sta SWACNT
         rts

silenceavoxvoice
         SPEAK avoxsilentdata
         rts
avoxsilentdata
         .byte 31,255
     endif

     ifconst HSSUPPORT
detectatarivoxeeprom
         ; do a test to see if atarivox eeprom can be accessed, and save results
         jsr AVoxDetect
         eor #$ff ; invert for easy 7800basic if...then logic
         sta avoxdetected
         rts 

detecthsc
         ; we check the first 2 bytes of the HSC signature...
         lda $1002
         cmp #$68
         bne detecthscfail
         lda $1003
         cmp #$83
         bne detecthscfail
         lda #$ff
         rts
detecthscfail
         lda #0
         rts
     endif ; HSSUPPORT

checkjoybuttons
     ;we poll the joystick fire buttons and throw them in shadow registers now
     lda #0
     sta sINPT1
     sta sINPT3
     lda INPT4
     bmi .skipp0firecheck
     ;one button joystick is down
     lda #$f0
     sta sINPT1
     lda joybuttonmode
     and #%00000100
     beq .skipp0firecheck
     lda joybuttonmode
     ora #%00000100
     sta joybuttonmode
     sta SWCHB
.skipp0firecheck
     lda INPT5
     bmi .skipp1firecheck
     ;one button joystick is down
     lda #$f0
     sta sINPT3
     lda joybuttonmode
     and #%00010000
     beq .skipp1firecheck
     lda joybuttonmode
     ora #%00010000
     sta joybuttonmode
     sta SWCHB
.skipp1firecheck
     lda INPT1
     ora sINPT1
     sta sINPT1

     lda INPT3
     ora sINPT3
     sta sINPT3
     rts

drawwait
     lda visibleover
     bne drawwait ; make sure the visible screen isn't being drawn
     rts

servicesfxchannels
     lda sfx1pointlo
     ora sfx1pointhi
     beq skipservicesfx1 ; (sfx1pointlo) isn't pointing at a sound

     lda sfx1tick
     beq servicesfx1_cont1
     dec sfx1tick ; frame countdown is non-zero. subtract one and
     jmp skipservicesfx1 ; skip playing the sound

servicesfx1_cont1

     lda sfx1frames ; set the frame countdown for this sound chunk
     sta sfx1tick

     lda sfx1priority ; decrease the sound's priority if its non-zero
     beq servicesfx1_cont2
     dec sfx1priority
servicesfx1_cont2

     ldy #0 ; play the sound
     lda (sfx1pointlo),y
     sta temp1
     clc
     adc sfx1poffset ; take into account any pitch modification
     sta AUDF0
     iny
     lda (sfx1pointlo),y
     sta AUDC0
     sta temp2
     iny
     lda (sfx1pointlo),y
     sta AUDV0

     ora temp2
     ora temp1 ; check if F|C|V=0
     beq zerosfx1 ; if so, we're at the end of the sound.

     ; advance the pointer to the next sound chunk
     clc
     lda sfx1pointlo
     adc #3
     sta sfx1pointlo
     lda sfx1pointhi
     adc #0
     sta sfx1pointhi
     jmp skipservicesfx1
     
zerosfx1
     sta sfx1pointlo
     sta sfx1pointhi
     sta sfx1priority
skipservicesfx1
     
     ifnconst TIASFXMONO
         lda sfx2pointlo
         ora sfx2pointhi
         beq skipservicesfx2 ; (sfx2pointlo) isn't pointing at a sound

         lda sfx2tick
         beq servicesfx2_cont1
         dec sfx2tick ; frame countdown is non-zero. subtract one and
         jmp skipservicesfx2 ; skip playing the sound

servicesfx2_cont1

         lda sfx2frames ; set the frame countdown for this sound chunk
         sta sfx2tick

         lda sfx2priority ; decrease the sound's priority if its non-zero
         beq servicesfx2_cont2
         dec sfx2priority
servicesfx2_cont2

         ldy #0 ; play the sound
         lda (sfx2pointlo),y
         sta temp1
         clc
         adc sfx2poffset ; take into account any pitch modification
         sta AUDF1
         iny
         lda (sfx2pointlo),y
         sta AUDC1
         sta temp2
         iny
         lda (sfx2pointlo),y
         sta AUDV1

         ora temp2
         ora temp1 ; check if F|C|V=0
         beq zerosfx2 ; if so, we're at the end of the sound.

         ; advance the pointer to the next sound chunk
         clc
         lda sfx2pointlo
         adc #3
         sta sfx2pointlo
         lda sfx2pointhi
         adc #0
         sta sfx2pointhi
         jmp skipservicesfx2
         
zerosfx2
         sta sfx2pointlo
         sta sfx2pointhi
         sta sfx2priority
skipservicesfx2
     endif ; TIASFXMONO

     ifnconst pokeysupport
         rts
     else
         ldx #6
checkpokeyplayingloop
         lda pokey1pointlo,x
         ora pokey1pointhi,x
         beq pokeychannelinactive
         jsr playpokeysfxA ; x=channel*2
pokeychannelinactive
         dex
         dex
         bpl checkpokeyplayingloop
         rts

playpokeysfxA
         txa
         tay
         lda pokey1tick,x
         beq playpokeysfxAcont
         sec
         sbc #1
         sta pokey1tick,x ; sound resolution is >1 frame, and we're mid-tock...
         rts

playpokeysfxAcont
         lda pokey1frames,x ; set the frame countdown for this sound chunk
         sta pokey1tick,x

         lda pokey1priority,x ; decrease the sound's priority if its non-zero
         beq playpokeysfxAcont2
         sec
         sbc #1
         sta pokey1priority,x
playpokeysfxAcont2


         ; *** FREQUENCY
         lda (pokey1pointlo,x)
         sta temp1
         clc
         adc pokey1offset,x ; take into account any pitch modification
         sta (pokeybaselo),y ; PAUDF0,0

         inc pokey1pointlo,x
         bne skippokeyhiinc1
         inc pokey1pointhi,x
skippokeyhiinc1

         ; *** WAVE
         lda (pokey1pointlo,x)
         asl
         asl
         asl
         asl ; x16

         inc pokey1pointlo,x
         bne skippokeyhiinc2
         inc pokey1pointhi,x
skippokeyhiinc2

         ora (pokey1pointlo,x)
         iny
         sta (pokeybaselo),y 

         ora temp1 ; check if F|C|V=0
         beq zeropokeypoint ; if so, we're at the end of the sound.

         ; advance the pointer to the next sound chunk
         inc pokey1pointlo,x
         bne skippokeyhiinc3
         inc pokey1pointhi,x
skippokeyhiinc3

         rts
         
zeropokeypoint
         sta pokey1pointlo,x
         sta pokey1pointhi,x
         sta pokey1priority,x
         rts 

     endif ; pokeysupport

schedulesfx
     ifnconst TIASFXMONO
         ; called with temp1=<data temp2=>data temp3=pitch offset
         ifconst pokeysupport
             ldy #0
             lda (temp1),y
             cmp #$20 ; TIA?
             bne scheduletiasfx
             jmp schedulepokeysfx
scheduletiasfx
         endif ; pokeysupport
         lda sfx1pointlo
         ora sfx1pointhi
         beq schedulesfx1 ;if channel 1 is idle, use it
         lda sfx2pointlo
         ora sfx2pointhi
         beq schedulesfx2 ;if channel 2 is idle, use it
         ; Both channels are scheduled. 
         ldy #1
         lda (temp1),y
         bne interruptsfx
         rts ; the new sound has 0 priority and both channels are busy. Skip playing it.
interruptsfx
         ;Compare which active sound has a lower priority. We'll interrupt the lower one.
         lda sfx1priority
         cmp sfx2priority
         bcs schedulesfx2
schedulesfx1
     endif ; TIASFXMONO

     ldy #1 ; get priority and sound-resolution (in frames)
     lda (temp1),y
     sta sfx1priority
     iny
     lda (temp1),y
     sta sfx1frames
     lda temp1
     clc
     adc #3
     sta sfx1pointlo
     lda temp2
     adc #0
     sta sfx1pointhi
     lda temp3
     sta sfx1poffset
     lda #0
     sta sfx1tick
     rts
     ifnconst TIASFXMONO
schedulesfx2
         ldy #1
         lda (temp1),y
         sta sfx2priority
         iny
         lda (temp1),y
         sta sfx2frames
         lda temp1
         clc
         adc #3
         sta sfx2pointlo
         lda temp2
         adc #0
         sta sfx2pointhi
         lda temp3
         sta sfx2poffset
         lda #0
         sta sfx2tick
         rts
     endif ; TIASFXMONO

     ifconst pokeysupport
schedulepokeysfx
         ldx #6
schedulepokeysfxloop
         lda pokey1pointlo,x
         ora pokey1pointhi,x
         bne schedulespokeysearch
         jmp schedulepokeyX ; we found an unused channel, so use it...
schedulespokeysearch
         dex
         dex
         bpl schedulepokeysfxloop

         ; if we're here, all 4 channels are presently playing a sound...
         ldy #1
         lda (temp1),y ; peek at the priority of this sfx...
         bne schedulepokeysfxcont1
         rts ; ...and skip it if it's 0 priority
schedulepokeysfxcont1

         ; figure out which current sound has the lowest priority...
         lda #0
         sta temp8
         lda pokey1priority
         sta temp9
         ldx #6
findlowprioritypokeyloop
         lda pokey1priority,x
         cmp temp9
         bcs findlowprioritypokeyloopcontinue
         sta temp9
         stx temp8
findlowprioritypokeyloopcontinue
         dex
         dex
         bne findlowprioritypokeyloop
         ldx temp8 ; the low priority channel we'll interrupt

schedulepokeyX
         ;called with X=2*pokey channel to play on...
         ldy #1 ; get priority and sound-resolution (in frames)
         lda (temp1),y
         sta pokey1priority,x
         iny
         lda (temp1),y
         sta pokey1frames,x

         lda temp1
         clc
         adc #3
         sta pokey1pointlo,x
         lda temp2
         adc #0
         sta pokey1pointhi,x
         lda temp3
         sta pokey1offset,x
         lda #0
         sta pokey1tick,x
         rts
     endif ; pokeysupport

plotsprite
     ifconst debugcolor
         lda #$00
         sta BACKGRND
     endif

plotspritewait
     lda visibleover
     bne plotspritewait

     ifconst debugcolor
         lda #$00
         sta BACKGRND
     endif

     ;arguments: 
     ; temp1=lo graphicdata 
     ; temp2=hi graphicdata 
     ; temp3=palette | width byte
     ; temp4=x
     ; temp5=y
     ; temp6=mode
     lda temp5 ;Y position
     lsr ; 2 - Divide by 8 or 16
     lsr ; 2
     lsr ; 2
     if WZONEHEIGHT = 16
         lsr ; 2
     endif

     tax

     ; the next block allows for vertical masking, and ensures we don't overwrite non-DL memory

     cmp #WZONECOUNT

     bcc continueplotsprite1 ; the sprite is fully on-screen, so carry on...
     ; otherwise, check to see if the bottom half is in zone 0...

     if WZONEHEIGHT = 16
         cmp #15
     else
         cmp #31
     endif

     bne exitplotsprite1
     ldx #0
     jmp continueplotsprite2
exitplotsprite1
     rts

continueplotsprite1

     lda DLPOINTL,x ;Get pointer to DL that this sprite starts in
     sta dlpnt
     lda DLPOINTH,x
     sta dlpnt+1

     ;Create DL entry for upper part of sprite

     ldy dlend,x ;Get the index to the end of this DL

     ifconst CHECKOVERWRITE
         cpy #DLLASTOBJ
         bne continueplotsprite1a
         rts
continueplotsprite1a
     endif

     lda temp1 ; graphic data, lo byte
     sta (dlpnt),y ;Low byte of data address

     iny
     lda temp6
     sta (dlpnt),y

     iny
     lda temp5 ;Y position

     if WZONEHEIGHT = 16
         and #$0F
     else ; WZONEHEIGHT = 8
         and #$7
     endif

     ora temp2 ; graphic data, hi byte
     sta (dlpnt),y

     iny
     lda temp3 ;palette|width
     sta (dlpnt),y

     iny
     lda temp4 ;Horizontal position
     sta (dlpnt),y

     iny
     sty dlend,x

     lda temp5

     and #(WZONEHEIGHT-1)

     beq doneSPDL ;branch if it is

     ;Create DL entry for lower part of sprite

     inx ;Next region

     cpx #WZONECOUNT

     bcc continueplotsprite2 ; the second half of the sprite is fully on-screen, so carry on...
     rts
continueplotsprite2

     lda DLPOINTL,x ;Get pointer to next DL
     sta dlpnt
     lda DLPOINTH,x
     sta dlpnt+1
     ldy dlend,x ;Get the index to the end of this DL

     ifconst CHECKOVERWRITE
         cpy #DLLASTOBJ
         bne continueplotsprite2a
         rts
continueplotsprite2a
     endif

     lda temp1 ; graphic data, lo byte
     sta (dlpnt),y

     iny
     lda temp6
     sta (dlpnt),y

     iny
     lda temp5 ;Y position

     if WZONEHEIGHT = 16
         and #$0F
         eor #$0F
     endif
     if WZONEHEIGHT = 8
         and #$07
         eor #$07
     endif

     sta temp9
     lda temp2 ; graphic data, hi byte
     clc
     sbc temp9
     sta (dlpnt),y

     iny
     lda temp3 ;palette|width
     sta (dlpnt),y

     iny
     lda temp4 ;Horizontal position
     sta (dlpnt),y

     iny
     sty dlend,x
doneSPDL
     rts

plotcharloop
     ; ** read from a data indirectly pointed to from temp8,temp9
     ; ** format is: lo_data, hi_data, palette|width, x, y
     ; ** format ends with lo_data | hi_data = 0
     ifconst debugcolor
         lda #$00
         sta BACKGRND
     endif
plotcharloopwait
     lda visibleover
     bne plotcharloopwait
plotcharlooploop
     ldy #0
     lda (temp8),y
     sta temp1
     iny
     lda (temp8),y
     sta temp2
     ora temp1
     bne plotcharloopcontinue
     ;the pointer=0, so return
     rts
plotcharloopcontinue
     iny
     lda (temp8),y
     sta temp3
     iny
     lda (temp8),y
     sta temp4
     iny
     lda (temp8),y
     ;sta temp5 ; not needed with our late entry.
     jsr plotcharactersskipentry
     lda temp8
     clc
     adc #5
     sta temp8
     lda temp9
     adc #0
     sta temp9
     jmp plotcharlooploop

plotcharacters
     ifconst debugcolor
         lda #$00
         sta BACKGRND
     endif
plotcharacterswait
     lda visibleover
     bne plotcharacterswait

     ;arguments: 
     ; temp1=lo charactermap
     ; temp2=hi charactermap
     ; temp3=palette | width byte
     ; temp4=x
     ; temp5=y

     lda temp5 ;Y position

plotcharactersskipentry

     ;ifconst ZONEHEIGHT
     ; if ZONEHEIGHT = 16
     ; and #$0F
     ; endif
     ; if ZONEHEIGHT = 8
     ; and #$1F
     ; endif
     ;else
     ; and #$0F
     ;endif

     tax
     lda DLPOINTL,x ;Get pointer to DL that the characters are in
     sta dlpnt
     lda DLPOINTH,x
     sta dlpnt+1

     ;Create DL entry for the characters

     ldy dlend,x ;Get the index to the end of this DL

     ifconst CHECKOVERWRITE
         cpy #DLLASTOBJ
         bne continueplotcharacters
         rts
continueplotcharacters
     endif

     lda temp1 ; character map data, lo byte
     sta (dlpnt),y ;(1) store low address

     iny
     lda charactermode 
     sta (dlpnt),y ;(2) store mode

     iny
     lda temp2 ; character map, hi byte
     sta (dlpnt),y ;(3) store high address

     iny
     lda temp3 ;palette|width
     sta (dlpnt),y ;(4) store palette|width

     iny
     lda temp4 ;Horizontal position
     sta (dlpnt),y ;(5) store horizontal position

     iny
     sty dlend,x ; save display list end byte
     rts

plotcharacterslive
     ; a version of plotcharacters that draws live and minimally disrupts the screen...

     ;arguments: 
     ; temp1=lo charactermap
     ; temp2=hi charactermap
     ; temp3=palette | width byte
     ; temp4=x
     ; temp5=y

     lda temp5 ;Y position

     tax
     lda DLPOINTL,x ;Get pointer to DL that the characters are in
     sta dlpnt
     lda DLPOINTH,x
     sta dlpnt+1

     ;Create DL entry for the characters

     ldy dlend,x ;Get the index to the end of this DL

     ifconst CHECKOVERWRITE
         cpy #DLLASTOBJ
         bne continueplotcharacterslive
         rts
continueplotcharacterslive
     endif

     lda temp1 ; character map data, lo byte
     sta (dlpnt),y ;(1) store low address

     iny
     ; we don't add the second byte yet, since the charmap could briefly
     ; render without a proper character map address, width, or position.
     lda charactermode 
     sta (dlpnt),y ;(2) store mode

     iny
     lda temp2 ; character map, hi byte
     sta (dlpnt),y ;(3) store high address

     iny
     lda temp3 ;palette|width
     sta (dlpnt),y ;(4) store palette|width

     iny
     lda temp4 ;Horizontal position
     sta (dlpnt),y ;(5) store horizontal position

     iny
     sty dlend,x ; save display list end byte

     rts

plotvalue
     ; calling 7800basic command:
     ; plotvalue digit_gfx palette variable/data number_of_digits screen_x screen_y
     ; ...displays the variable as BCD digits
     ;
     ; asm sub arguments: 
     ; temp1=lo charactermap
     ; temp2=hi charactermap
     ; temp3=palette | width byte
     ; temp4=x
     ; temp5=y
     ; temp6=number of digits
     ; temp7=lo variable
     ; temp8=hi variable
     ; temp9=character mode

plotdigitcount     = temp6

     lda #0
     tay
     ldx valbufend

     lda plotdigitcount
     and #1
     beq pvnibble2char
     lda #0
     sta VALBUFFER,x ; just in case we skip this digit
     beq pvnibble2char_skipnibble

pvnibble2char
     ; high nibble...
     lda (temp7),y
     and #$f0 
     lsr
     lsr
     lsr
     ifnconst DOUBLEWIDE ; multiply value by 2 for double-width
         lsr
     endif

     clc
     adc temp1 ; add the offset to character graphics to our value
     sta VALBUFFER,x
     inx
     dec plotdigitcount

pvnibble2char_skipnibble
     ; low nibble...
     lda (temp7),y
     and #$0f 
     ifconst DOUBLEWIDE ; multiply value by 2 for double-width
         asl
     endif
     clc
     adc temp1 ; add the offset to character graphics to our value
     sta VALBUFFER,x 
     inx
     iny

     dec plotdigitcount
     bne pvnibble2char

     ;point to the start of our valuebuffer
     clc
     lda #<VALBUFFER
     adc valbufend
     sta temp1
     lda #>VALBUFFER
     adc #0
     sta temp2

     ;advance valbufend to the end of our value buffer
     stx valbufend

     ifnconst plotvalueonscreen
         jmp plotcharacters
     else
         jmp plotcharacterslive
     endif


plotvalueextra
     ; calling 7800basic command:
     ; plotvalue digit_gfx palette variable/data number_of_digits screen_x screen_y
     ; ...displays the variable as BCD digits
     ;
     ; asm sub arguments: 
     ; temp1=lo charactermap
     ; temp2=hi charactermap
     ; temp3=palette | width byte
     ; temp4=x
     ; temp5=y
     ; temp6=number of digits
     ; temp7=lo variable
     ; temp8=hi variable

     lda #0
     tay
     ldx valbufend
     ifnconst plotvalueonscreen
         sta VALBUFFER,x
     endif

     lda plotdigitcount
     and #1
     
     bne pvnibble2char_skipnibbleextra

pvnibble2charextra
     ; high nibble...
     lda (temp7),y
     and #$f0 
     lsr
     lsr
     ifnconst DOUBLEWIDE ; multiply value by 2 for double-width
         lsr
     endif
     clc
     adc temp1 ; add the offset to character graphics to our value
     sta VALBUFFER,x
     inx

     ; second half of the digit
     clc
     adc #1
     sta VALBUFFER,x
     inx

pvnibble2char_skipnibbleextra
     ; low nibble...
     lda (temp7),y
     and #$0f 
     ifconst DOUBLEWIDE ; multiply value by 2 for double-width
         asl
     endif
     asl

     clc
     adc temp1 ; add the offset to character graphics to our value
     sta VALBUFFER,x 
     inx

     clc
     adc #1
     sta VALBUFFER,x
     inx
     iny

     dec plotdigitcount
     bne pvnibble2charextra

     ;point to the start of our valuebuffer
     clc
     lda #<VALBUFFER
     adc valbufend
     sta temp1
     lda #>VALBUFFER
     adc #0
     sta temp2

     ;advance valbufend to the end of our value buffer
     stx valbufend

     ifnconst plotvalueonscreen
         jmp plotcharacters
     else
         jmp plotcharacterslive
     endif

boxcollision
     ; prior to getting here...
     ; we have 4x lda #immed, 4x lda zp, 8x sta zp, and 1x jsr
     ; = 50 cycles to start. eep!

__boxx1 = temp1
__boxy1 = temp2
__boxw1 = temp3
__boxh1 = temp4

__boxx2 = temp5
__boxy2 = temp6
__boxw2 = temp7
__boxh2 = temp8

     ifconst collisionwrap
         ; if collision durring screen wrap is required, we add constants to
         ; coordinates, so we're not looking at negatives. (if reasonably
         ; sized sprites are used, anyway.)
         lda __boxx1 ;3 
         clc ;2
         adc #48 ;2
         sta __boxx1 ;3

         lda __boxx2 ;3
         clc ;2
         adc #48 ;2
         sta __boxx2 ;3

         lda __boxy1 ;3
         clc ;2
         ;adc #32 ;2
         adc #((256-WSCREENHEIGHT)/2) ;2
         sta __boxy1 ;3

         lda __boxy2 ;3
         clc ;2
         ;adc #32 ;2
         adc #((256-WSCREENHEIGHT)/2) ;2
         sta __boxy2 ;3
         ;=== +40 cycles for using collisionwrap
     endif

DoXCollisionCheck
     lda __boxx1 ;3
     cmp __boxx2 ;3
     bcs X1isbiggerthanX2 ;2/3
X2isbiggerthanX1
     adc __boxw1 ;3
     cmp __boxx2 ;3
     bcs DoYCollisionCheck ;3/2
     rts ;6 - carry clear, no collision
X1isbiggerthanX2
     clc ;2
     sbc __boxw2 ;3
     cmp __boxx2 ;3
     bcs noboxcollision ;3/2
DoYCollisionCheck
     lda __boxy1 ;3
     cmp __boxy2 ;3
     bcs Y1isbiggerthanY2 ;3/2
Y2isbiggerthanY1
     adc __boxh1 ;3
     cmp __boxy2 ;3
     rts ;6 
Y1isbiggerthanY2
     clc ;2
     sbc __boxh2 ;3
     cmp __boxy2 ;3
     bcs noboxcollision ;3/2
yesboxcollision
     sec ;2
     rts ;6
noboxcollision
     clc ;2
     rts ;6

     ;==48 cycles worst case for checks+rts
     ;TOTAL =~48+50 for call+checks+rts
     ;TOTAL =~48+50+40 for call+wrap+checks+rts

randomize
     lda rand
     lsr
     rol rand16
     bcc noeor
     eor #$B4
noeor
     sta rand
     eor rand16
     rts

     ; bcd conversion routine courtesy Omegamatrix
     ; http://atariage.com/forums/blog/563/entry-10832-hex-to-bcd-conversion-0-99/
converttobcd
     ;value to convert is in the accumulator
     sta temp1
     lsr
     adc temp1
     ror
     lsr
     lsr
     adc temp1
     ror
     adc temp1
     ror
     lsr
     and #$3C
     sta temp2
     lsr
     adc temp2
     adc temp1 
     rts ; return the result in the accumulator

     ; Y and A contain multiplicands, result in A
mul8
     sty temp1
     sta temp2
     lda #0
reptmul8
     lsr temp2
     bcc skipmul8
     clc
     adc temp1
     ;bcs donemul8 might save cycles?
skipmul8
     ;beq donemul8 might save cycles?
     asl temp1
     bne reptmul8
donemul8
     rts

div8
     ; A=numerator Y=denominator, result in A
     cpy #2
     bcc div8end+1;div by 0 = bad, div by 1=no calc needed, so bail out
     sty temp1
     ldy #$ff
div8loop
     sbc temp1
     iny
     bcs div8loop
div8end
     tya
     ; result in A
     rts

     ; Y and A contain multiplicands, result in temp2,A=low, temp1=high
mul16
     sty temp1
     sta temp2

     lda #0
     ldx #8
     lsr temp1
mul16_1
     bcc mul16_2
     clc
     adc temp2
mul16_2
     ror
     ror temp1
     dex
     bne mul16_1
     sta temp2
     rts

     ; div int/int
     ; numerator in A, denom in temp1
     ; returns with quotient in A, remainder in temp1
div16
     sty temp1
     ldx #8
loopdiv
     cmp temp1
     bcc toosmalldiv
     sbc temp1 ; Note: Carry is, and will remain, set.
     rol temp2
     rol
     dex
     bne loopdiv
     beq donediv
toosmalldiv
     rol temp2
     rol
     dex
     bne loopdiv
donediv
     sta temp1
     lda temp2
     rts

     ifconst bankswitchmode
BS_jsr
         ifconst MCPDEVCART
             ora #$18
             sta $3000
         else
             sta $8000
         endif
         pla
         tax
         pla
         rts

BS_return
         pla ; bankswitch bank
         ifconst BANKRAM
             sta currentbank
             ora currentrambank
         endif
         ifconst MCPDEVCART
             ora #$18
             sta $3000
         else
             sta $8000
         endif
         pla ; bankswitch $0 flag
         rts 
     endif

     ifconst pokeysupport
         ; pokey detection routine. we check for pokey in the XBOARD/XM location, 
         ; and the standard $4000 location.
         ; if pokey the pokey is present, this routine will reset it.

detectpokeylocation
checklocation1
         ;XBoard/XM...
         lda #%00010000
         sta $470 ; tell XCTRL to hit the POKEY enable
         lda #<($450)
         sta pokeybaselo
         lda #>($450)
         sta pokeybasehi
         jsr checkforpokey
         lda pokeydetected
         bne checklocation2
         eor #$ff ; invert state for 7800basic if...then test
         sta pokeydetected
         rts

checklocation2
         ;traditional...
         lda #<($4000)
         sta pokeybaselo
         lda #>($4000)
         sta pokeybasehi
         jsr checkforpokey
         eor #$ff ; invert state for 7800basic if...then test
         sta pokeydetected
         rts

checkforpokey
         ldy #$0f
         lda #$00
         sta pokeydetected
resetpokeyregistersloop
         sta (pokeybase),y
         dey 
         bpl resetpokeyregistersloop

         ldy #PAUDCTL
         sta (pokeybase),y
         ldy #PSKCTL
         sta (pokeybase),y

         ; let the dust settle...
         nop
         nop
         nop

         ; we're in reset, so the RANDOM register should read $ff...
         ldy #PRANDOM
         lda (pokeybase),y
         cmp #$ff
         bne nopokeydetected

         ; take pokey out of reset...
         ldy #PSKCTL
         lda #3
         sta (pokeybase),y
         ldy #PAUDCTL
         lda #0
         sta (pokeybase),y

         ; let the dust settle again...
         nop
         nop
         nop

         ; we're out of reset, so RANDOM should read non-$ff...
         ldy #PRANDOM
         lda (pokeybase),y
         cmp #$ff
         beq checkpokeyoncemore ;one $ff might be a fluke. try again...
         rts

checkpokeyoncemore
         lda (pokeybase),y
         cmp #$ff
         beq nopokeydetected
         rts

nopokeydetected
         dec pokeydetected
         rts
     endif

checkselectswitch
     lda SWCHB ; first check the real select switch...
     and #%00000010
     beq checkselectswitchreturn ; switch is pressed
     lda SWCHA ; then check the soft "select" joysick code...
     and #%10110000 ; R_DU
checkselectswitchreturn
     rts

checkresetswitch
     lda SWCHB ; first check the real reset switch...
     and #%00000001
     beq checkresetswitchreturn ; switch is pressed
     lda SWCHA ; then check the soft "reset" joysick code...
     and #%01110000 ; _LDU
checkresetswitchreturn
     rts

createallgamedlls
     ldx #0
     lda #NVLINES
     jsr createnonvisibledlls
     stx temp2
     jsr createvisiblelines
     stx temp3
createallgamedllscontinue
     lda #(NVLINES+51) ; +51 to allow for PAL display without problems
     jsr createnonvisibledlls

     ldx temp2
     lda DLLMEM,x
     ora #%10000000 ; trigger nmi as the first visible starts
     sta DLLMEM,x

     ldx temp3
     lda DLLMEM,x
     ora #%10000000 ; trigger nmi as the last visible line finishes
     sta DLLMEM,x
     rts

createnonvisibledlls
     sta temp1
createnonvisibledllsloop
     sec
     sbc #16
     bcc createlastdll
     sta temp1
     lda #%01001111 ;low nibble=16 lines, high nibble=Holey DMA
createlastdllreturn
     sta DLLMEM,x
     inx
     lda #$21 ; blank
     sta DLLMEM,x
     inx
     lda #$00
     sta DLLMEM,x
     inx
     lda temp1
     bne createnonvisibledllsloop
     rts
createlastdll
     ldy temp1
     dey
     lda #0
     sta temp1
     tya
     ora #%01000000 ; low nibble=remaining lines, high nibble=Holey DMA
     jmp createlastdllreturn

createvisiblelines
     ldy #0
createvisiblelinesloop
     lda.w DLHEIGHT,y
     ora #(WZONEHEIGHT * 4) ; set Holey DMA for 8 or 16 tall zones

     sta DLLMEM,x
     inx
     lda DLPOINTH,y
     sta DLLMEM,x
     inx
     lda DLPOINTL,y
     sta DLLMEM,x
     inx
     iny
     cpy #WZONECOUNT
     bne createvisiblelinesloop
     rts

waitforvblankstart
visibleoverwait
     BIT MSTAT
     bpl visibleoverwait
vblankstartwait
     BIT MSTAT
     bmi vblankstartwait
     rts

     ifconst HSSUPPORT 
         ifnconst hiscorefont
             echo ""
             echo "WARNING: High score support is enabled, but the hiscorefont.png was"
             echo " NOT imported with incgraphic. The high score display code"
             echo " has been omitted from this build."
             echo ""
         else
hscdrawscreen

HSSCOREY             = 6

 ifconst HSSCORESIZE
SCORESIZE = HSSCORESIZE
 else
SCORESIZE = 6
 endif

             ;save shadow registers for later return...
             lda sCTRL
             sta ssCTRL
             lda sCHARBASE
             sta ssCHARBASE 
             jsr drawwait
             jsr blacken320colors
             jsr clearscreen

             ;set the character base to the HSC font
             lda #>hiscorefont
             sta CHARBASE
             sta sCHARBASE

             lda #%01000011 ;Enable DMA, mode=320A
             sta CTRL
             sta sCTRL

             lda #60
             sta hsjoydebounce

             lda #0
             sta hscursorx
             sta framecounter
             ifnconst HSCOLORCHASESTART
                 lda #$8D ; default is blue. why not?
             else
                 lda #HSCOLORCHASESTART
             endif
             sta hscolorchaseindex

             lda #$0F
             sta P0C2 ; base text is white

             jsr hschasecolors

             ; ** plot all of the initials
             lda #<HSRAMInitials
             sta temp1 ; charmaplo
             lda #>HSRAMInitials
             sta temp2 ; charmaphi
             lda #32+29 ; palette=0-29 | 32-(width=3)
             sta temp3 ; palette/width
             lda #104
             sta temp4 ; X
             lda #(HSSCOREY/(WZONEHEIGHT/8))
             sta temp5 ; Y
plothsinitialsloop
             jsr plotcharacters
             clc
             lda temp3
             adc #32
             sta temp3
             inc temp5
             if WZONEHEIGHT = 8
                 inc temp5
             endif
             clc
             lda #3
             adc temp1
             sta temp1
             cmp #(<(HSRAMInitials+15))
             bcc plothsinitialsloop

             ifconst HSGAMENAMELEN
                 ;plot the game name...
                 lda #<HSGAMENAMEtable
                 sta temp1 ; charmaplo
                 lda #>HSGAMENAMEtable
                 sta temp2 ; charmaphi
                 lda #(32-HSGAMENAMELEN) ; palette=0*29 | 32-(width=3)
                 sta temp3 ; palette/width
                 lda #(80-(HSGAMENAMELEN*2))
                 sta temp4 ; X
                 lda #(0/(WZONEHEIGHT/8))
                 sta temp5 ; Y
                 jsr plotcharacters
             endif

             ;plot "difficulty"...
             ldy gamedifficulty
             ifnconst HSNOLEVELNAMES
                 lda highscoredifficultytextlo,y
                 sta temp1
                 lda highscoredifficultytexthi,y
                 sta temp2
                 sec
                 lda #32
                 sbc highscoredifficultytextlen,y
                 sta temp3 ; palette/width
                 sec
                 lda #40
                 sbc highscoredifficultytextlen,y
                 asl
                 sta temp4 ; X
             else
                 lda #<HSHIGHSCOREStext
                 sta temp1 ; charmaplo
                 lda #>HSHIGHSCOREStext
                 sta temp2 ; charmaphi
                 lda #(32-11) ; palette=0*29 | 32-(width=3)
                 sta temp3 ; palette/width
                 lda #(80-(11*2))
                 sta temp4 ; X
             endif

             lda #(2/(WZONEHEIGHT/8))
             sta temp5 ; Y
             jsr plotcharacters

             ldy hsdisplaymode ; 0=attact mode, 1=player eval, 2=player 1 eval, 3=player 2 player eval
             bne carronwithscoreevaluation
             jmp donoscoreevaluation
carronwithscoreevaluation
             dey
             lda highscorelabeltextlo,y
             sta temp1
             lda highscorelabeltexthi,y
             sta temp2
             sec
             lda #(32-15) ; palette=0*29 | 32-(width=3)
             sta temp3 ; palette/width
             lda highscorelabeladjust1,y
             sta temp4 ; X
             lda #(18/(WZONEHEIGHT/8))
             sta temp5 ; Y
             jsr plotcharacters

             ldy hsdisplaymode ; 0=attact mode, 1=player eval, 2=player 1 eval, 3=player 2 player eval
             dey

             ;plot the current player score...
             lda #(32-SCORESIZE) ; palette=0*32 
             sta temp3 ; palette/width
             lda highscorelabeladjust2,y
             sta temp4 ; X
             lda #(18/(WZONEHEIGHT/8))
             sta temp5 ; Y

             lda scorevarlo,y
             sta temp7 ; score variable lo
             lda scorevarhi,y
             sta temp8 ; score variable hi

             lda #(hiscorefont_mode | %01100000) ; charactermode 
             sta temp9

             lda #<(hiscorefont+33) ; +33 to get to '0' character
             sta temp1 ; charmaplo
             lda #>(hiscorefont+33)
             sta temp2 ; charmaphi
             lda #SCORESIZE
             sta temp6
             jsr plotvalue

             ifconst HSGAMERANKS
                 
                 ldx #$ff ; start at 0 after the inx...
comparescore2rankloop
                 inx
                 ldy #0
                 lda rankvalue_0,x
                 cmp (temp7),y
                 bcc score2rankloopdone
                 bne comparescore2rankloop
                 iny
                 lda rankvalue_1,x
                 cmp (temp7),y
                 bcc score2rankloopdone
                 bne comparescore2rankloop
                 iny
                 lda (temp7),y
                 cmp rankvalue_2,x
                 bcs score2rankloopdone
                 jmp comparescore2rankloop
score2rankloopdone
                 stx hsnewscorerank
                 
                 lda ranklabello,x
                 sta temp1
                 lda ranklabelhi,x
                 sta temp2
                 sec
                 lda #32 ; palette=0*29 | 32-(width=3)
                 sbc ranklabellengths,x
                 sta temp3 ; palette/width
                 sec
                 lda #(40+6)
                 sbc ranklabellengths,x
                 asl
                 sta temp4 ; X
                 lda #(20/(WZONEHEIGHT/8))
                 sta temp5 ; Y
                 jsr plotcharacters

                 ldx hsnewscorerank

                 lda #<highscoreranklabel
                 sta temp1
                 lda #>highscoreranklabel
                 sta temp2

                 lda #(32-5) ; palette=0*29 | 32-(width=3)
                 sta temp3 ; palette/width
                 lda #(40-6)
                 sec
                 sbc ranklabellengths,x
                 asl
                 sta temp4 ; X
                 lda #(20/(WZONEHEIGHT/8))
                 sta temp5 ; Y
                 jsr plotcharacters
             endif


             ; ** which line did this player beat?
             lda #$ff
             sta hsnewscoreline
             ldx #$fd 
comparescoreadd2x
             inx
comparescoreadd1x
             inx
comparescore2lineloop
             inc hsnewscoreline
             inx ; initialrun, x=0
             cpx #15
             beq nohighscoreforyou
             ldy #0
             lda HSRAMScores,x
             cmp (temp7),y ; first score digit
             bcc score2lineloopdonedel1x
             bne comparescoreadd2x
             iny
             inx
             lda HSRAMScores,x
             cmp (temp7),y
             bcc score2lineloopdonedel2x
             bne comparescoreadd1x
             iny
             inx
             lda (temp7),y
             cmp HSRAMScores,x
             bcs score2lineloopdonedel3x
             jmp comparescore2lineloop
nohighscoreforyou
             lda #$ff
             sta hsnewscoreline
             sta countdownseconds
             jmp donoscoreevaluation
score2lineloopdonedel3x
             dex
score2lineloopdonedel2x
             dex
score2lineloopdonedel1x
             dex

             ; 0 1 2
             ; 3 4 5
             ; 6 7 8
             ; 9 0 1
             ; 2 3 4

             stx temp9
             cpx #11
             beq postsortscoresuploop
             ldx #11
sortscoresuploop
             lda HSRAMScores,x
             sta HSRAMScores+3,x
             lda HSRAMInitials,x
             sta HSRAMInitials+3,x
             dex
             cpx temp9
             bne sortscoresuploop
postsortscoresuploop

             ;stick the score and cleared initials in the slot...
             inx
             ldy #0 
             sty hsinitialhold
             lda (temp7),y
             sta HSRAMScores,x
             iny
             lda (temp7),y
             sta HSRAMScores+1,x
             iny
             lda (temp7),y
             sta HSRAMScores+2,x
             lda #0
             sta HSRAMInitials,x
             lda #29
             sta HSRAMInitials+1,x
             sta HSRAMInitials+2,x

             stx hsinitialpos

donoscoreevaluation

             lda #(32+(32-SCORESIZE)) ; palette=0*32 | 32-(width=6)
             sta temp3 ; palette/width
             lda #(72+(4*(6-SCORESIZE)))
             sta temp4 ; X
             lda #(HSSCOREY/(WZONEHEIGHT/8))
             sta temp5 ; Y
             lda #<HSRAMScores
             sta temp7 ; score variable lo
             lda #>HSRAMScores
             sta temp8 ; score variable hi
             lda #(hiscorefont_mode | %01100000) ; charactermode 
             sta temp9
plothsscoresloop
             lda #<(hiscorefont+33) ; +33 to get to '0' character
             sta temp1 ; charmaplo
             lda #>(hiscorefont+33)
             sta temp2 ; charmaphi
             lda #6
             sta temp6
             jsr plotvalue
             clc
             lda temp3
             adc #32
             sta temp3
             inc temp5
             if WZONEHEIGHT = 8
                 inc temp5
             endif
             clc
             lda #3
             adc temp7
             sta temp7
             cmp #(<(HSRAMScores+15))
             bcc plothsscoresloop
plothsindex
             lda #32+31 ; palette=0*32 | 32-(width=1)
             sta temp3 ; palette/width
             lda #44
             sta temp4 ; X
             lda #(HSSCOREY/(WZONEHEIGHT/8))
             sta temp5 ; Y
             lda #<hsgameslotnumbers
             sta temp7 ; score variable lo
             lda #>hsgameslotnumbers
             sta temp8 ; score variable hi
             lda #(hiscorefont_mode | %01100000) ; charactermode 
             sta temp9
plothsindexloop
             lda #<(hiscorefont+33)
             sta temp1 ; charmaplo
             lda #>(hiscorefont+33)
             sta temp2 ; charmaphi
             lda #1
             sta temp6 ; number of characters
             jsr plotvalue
             clc
             lda temp3
             adc #32
             sta temp3
             inc temp5
             if WZONEHEIGHT = 8
                 inc temp5
             endif
             inc temp7
             lda temp7
             cmp #(<(hsgameslotnumbers+5))
             bcc plothsindexloop

             jsr savescreen

             ifnconst HSSECONDS
                 lda #6
             else
                 lda #HSSECONDS
             endif
             sta countdownseconds
             
keepdisplayinghs
             jsr restorescreen


             jsr setuphsinpt1

             lda hsnewscoreline
             bpl carryonkeepdisplayinghs
             jmp skipenterscorecontrol
carryonkeepdisplayinghs


             ifnconst HSSECONDS
                 lda #6
             else
                 lda #HSSECONDS
             endif
             sta countdownseconds

             ;plot the cursor underneath the line
             lda #<(hiscorefont+28)
             sta temp1
             lda #>(hiscorefont+28)
             sta temp2
             lda #31 ; palette=0*32 | 32-(width=1)
             sta temp3 ; palette/width
             lda hscursorx
             asl
             asl
             clc
             adc #104
             sta temp4 ; X
             lda hsnewscoreline
             asl
             asl
             asl
             asl
             adc #((3*16)+6)
             sta temp5 ; Y
             lda #%01000000
             sta temp6
             jsr plotsprite

             lda SWCHA
             cpx #3
             bne hsskipadjustjoystick1
             asl
             asl
             asl
             asl
hsskipadjustjoystick1
             sta hsswcha
             and #%00110000
             cmp #%00110000
             beq hsjoystickskipped
             lda hsjoydebounce
             beq hsdontdebounce
             jmp hspostjoystick
hsdontdebounce
             ldx #1 ; small tick sound
             jsr playhssfx
             lda hsswcha
             and #%00110000
             ldx hscursorx
             cmp #%00100000 ; check down
             bne hsjoycheckup
             ldy hsinitialhold
             cpx #0
             bne skipavoid31_1
             cpy #0
             bne skipavoid31_1
             dey
skipavoid31_1
             dey
             jmp hssetdebounce
hsjoycheckup
             cmp #%00010000 ; check up
             bne hsjoystickskipped
             ldy hsinitialhold
             cpx #0
             bne skipavoid31_2
             cpy #15
             bne skipavoid31_2
             iny
skipavoid31_2
             iny
hssetdebounce
             tya
             and #31
             sta hsinitialhold
             lda #15
             sta hsjoydebounce
             bne hspostjoystick
hsjoystickskipped
             ; check the fire button only when the stick isn't engaged
             lda hsinpt1
             bpl hsbuttonskipped
             lda hsjoydebounce
             beq hsfiredontdebounce
             bne hspostjoystick
hsfiredontdebounce
             lda hsinitialhold
             cmp #31
             beq hsmovecursorback
             inc hscursorx
             inc hsinitialpos
             lda hscursorx
             cmp #3
             bne skiphsentryisdone
             lda #0
             sta framecounter
             lda #$ff
             sta hsnewscoreline
             dec hsinitialpos
             bne skiphsentryisdone
hsmovecursorback
             lda #29
             ldx hsinitialpos
             sta HSRAMInitials,x
             dec hsinitialpos
             dec hscursorx
             dex
             lda HSRAMInitials,x
             sta hsinitialhold

skiphsentryisdone
             ldx #0
             jsr playhssfx
             lda #20
             sta hsjoydebounce
             bne hspostjoystick

hsbuttonskipped
             lda #0
             sta hsjoydebounce
hspostjoystick

             ldx hsinitialpos
             lda hsinitialhold
             sta HSRAMInitials,x

             jmp skiphschasecolors

skipenterscorecontrol
             jsr hschasecolors
             jsr setuphsinpt1
             lda hsjoydebounce
             bne skiphschasecolors
             lda hsinpt1
             bmi returnfromhs
skiphschasecolors

             jsr drawscreen

             lda countdownseconds
             beq returnfromhs
             jmp keepdisplayinghs
returnfromhs
             jsr drawwait
             jsr clearscreen
             lda #0
             ldy #7
             jsr blacken320colors
             lda ssCTRL
             sta sCTRL
             lda ssCHARBASE
             sta sCHARBASE 
             rts

setuphsinpt1
             lda #$ff
             sta hsinpt1
             lda hsjoydebounce
             beq skipdebounceadjust
             dec hsjoydebounce
             bne skipstorefirebuttonstatus
skipdebounceadjust
             ldx hsdisplaymode
             cpx #3
             bne hsskipadjustjoyfire1
             lda sINPT3
             jmp hsskipadjustjoyfire1done
hsskipadjustjoyfire1
             lda sINPT1
hsskipadjustjoyfire1done
             sta hsinpt1
skipstorefirebuttonstatus
             rts



blacken320colors
             ldy #7
blacken320colorsloop
             sta P0C2,y
             dey
             bpl blacken320colorsloop
             rts

hschasecolors
             lda framecounter
             and #3
             bne hschasecolorsreturn
             inc hscolorchaseindex
             lda hscolorchaseindex
             
             sta P5C2
             sbc #$02
             sta P4C2
             sbc #$02
             sta P3C2
             sbc #$02
             sta P2C2
             sbc #$02
             sta P1C2
hschasecolorsreturn
             rts


playhssfx
             lda hssfx_lo,x
             sta temp1
             lda hssfx_hi,x
             sta temp2
             lda #0
             sta temp3
             jmp schedulesfx

hssfx_lo
             .byte <sfx_hsletterpositionchange, <sfx_hslettertick
hssfx_hi
             .byte >sfx_hsletterpositionchange, >sfx_hslettertick

sfx_hsletterpositionchange
             .byte $10,$18,$00
             .byte $02,$06,$08
             .byte $02,$06,$04
             .byte $00,$00,$00

sfx_hslettertick
             .byte $10,$18,$00
             .byte $00,$00,$0a
             .byte $00,$00,$00

highscorelabeladjust1
             .byte (80-(14*2)-(SCORESIZE*2)),(80-(16*2)-(SCORESIZE*2)),(80-(16*2)-(SCORESIZE*2))
highscorelabeladjust2
             .byte (80+(14*2)-(SCORESIZE*2)),(80+(16*2)-(SCORESIZE*2)),(80+(16*2)-(SCORESIZE*2))

scorevarlo
             .byte <(score0+((6-SCORESIZE)/2)),<(score0+((6-SCORESIZE)/2)),<(score1+((6-SCORESIZE)/2))
scorevarhi
             .byte >(score0+((6-SCORESIZE)/2)),>(score0+((6-SCORESIZE)/2)),>(score1+((6-SCORESIZE)/2))

             ifnconst HSNOLEVELNAMES
highscoredifficultytextlo
                 .byte <easylevelname, <mediumlevelname, <hardlevelname, <expertlevelname
highscoredifficultytexthi
                 .byte >easylevelname, >mediumlevelname, >hardlevelname, >expertlevelname

             ifnconst HSCUSTOMLEVELNAMES
highscoredifficultytextlen
                 .byte 22, 30, 26, 24
                 
easylevelname
                 .byte $04,$00,$12,$18,$1d,$0b,$04,$15,$04,$0b,$1d,$07,$08,$06,$07,$1d,$12,$02,$0e,$11,$04,$12
mediumlevelname
                 .byte $08,$0d,$13,$04,$11,$0c,$04,$03,$08,$00,$13,$04,$1d,$0b,$04,$15,$04,$0b,$1d,$07,$08,$06,$07,$1d,$12,$02,$0e,$11,$04,$12
hardlevelname
                 .byte $00,$03,$15,$00,$0d,$02,$04,$03,$1d,$0b,$04,$15,$04,$0b,$1d,$07,$08,$06,$07,$1d,$12,$02,$0e,$11,$04,$12
expertlevelname
                 .byte $04,$17,$0f,$04,$11,$13,$1d,$0b,$04,$15,$04,$0b,$1d,$07,$08,$06,$07,$1d,$12,$02,$0e,$11,$04,$12
             else
                 include "7800hsgamediffnames.asm"
             endif ; HSCUSTOMLEVELNAMES
             else
HSHIGHSCOREStext
                 .byte $07,$08,$06,$07,$1d,$12,$02,$0e,$11,$04,$12

             endif ; HSNOLEVELNAMES

highscorelabeltextlo
             .byte <player0label, <player1label, <player2label
highscorelabeltexthi
             .byte >player0label, >player1label, >player2label

player0label
             .byte $0f,$0b,$00,$18,$04,$11,$1d,$12,$02,$0e,$11,$04,$1a,$1d,$1d

player1label
             .byte $0f,$0b,$00,$18,$04,$11,$1d,$22,$1d,$12,$02,$0e,$11,$04,$1a

player2label
             .byte $0f,$0b,$00,$18,$04,$11,$1d,$23,$1d,$12,$02,$0e,$11,$04,$1a


             ifconst HSGAMENAMELEN
HSGAMENAMEtable
                 include "7800hsgamename.asm"
             endif
             ifconst HSGAMERANKS
                 include "7800hsgameranks.asm"
highscoreranklabel
                 .byte $11,$00,$0d,$0a,$1a
             endif

             ;ensure our table doesn't wrap a page...
             if ((<*)>251)
                 align 256
             endif
hsgameslotnumbers
             .byte 33,34,35,36,37
         endif

loaddifficultytable
         lda gamedifficulty
         and #$03 ; ensure the user hasn't selected an invalid difficulty
         sta gamedifficulty
         cmp hsdifficulty; check game difficulty is the same as RAM table
         bne loaddifficultytablecontinue1
         rts ; this high score difficulty table is already loaded
loaddifficultytablecontinue1
         lda gamedifficulty
         sta hsdifficulty
         ;we need to check the device for the table
         lda hsdevice
         bne loaddifficultytablecontinue2
         ; there's no save device. clear out this table.
         jmp cleardifficultytablemem
loaddifficultytablecontinue2
         lda hsdevice
         and #1
         beq memdeviceisntHSC
         jmp loaddifficultytableHSC
memdeviceisntHSC
         jmp loaddifficultytableAVOX

savedifficultytable
         ;*** we need to check wich device we should use...
         lda hsdevice
         bne savedifficultytablerealdevice
         rts ; its a ram device
savedifficultytablerealdevice
         and #1
         beq savememdeviceisntHSC
         jmp savedifficultytableHSC
savememdeviceisntHSC
         jmp savedifficultytableAVOX

savedifficultytableAVOX
         ; the load call already setup the memory structure and atarivox memory location
         jsr savealoadedHSCtablecontinue
         lda #HSIDHI
         sta eeprombuffer
         lda #HSIDLO
         sta eeprombuffer+1
         lda hsdifficulty
         sta eeprombuffer+2
         lda #32
         jsr AVoxWriteBytes
         rts

savedifficultytableHSC
         ;we always load a table before reaching here, so the
         ;memory structures from the load should be intact...
         ldy hsgameslot
         bpl savealoadedHSCtable
         rts
savealoadedHSCtable
         lda HSCGameDifficulty,y 
         cmp #$7F
         bne savealoadedHSCtablecontinue
         jsr initializeHSCtableentry
savealoadedHSCtablecontinue
         ;convert our RAM table to HSC format and write it out...
         ldy #0
         ldx #0
savedifficultytableScores

         lda HSRAMInitials,x
         sta temp3
         lda HSRAMInitials+1,x
         sta temp4
         lda HSRAMInitials+2,x
         sta temp5
         jsr encodeHSCInitials ; takes 3 byte initials from temp3,4,5 and stores 2 byte initials in temp1,2

         lda temp1
         sta (HSGameTableLo),y
         iny
         lda temp2
         sta (HSGameTableLo),y
         iny
         
         lda HSRAMScores,x
         sta (HSGameTableLo),y
         iny
         lda HSRAMScores+1,x
         sta (HSGameTableLo),y
         iny
         lda HSRAMScores+2,x
         sta (HSGameTableLo),y
         iny
         inx
         inx
         inx ; +3
         cpx #15
         bne savedifficultytableScores
         rts

loaddifficultytableHSC
         ; routine responsible for loading the difficulty table from HSC
         jsr findindexHSC
         ldy hsgameslot
         lda HSCGameDifficulty,y 
         cmp #$7F
         bne loaddifficultytableHSCcontinue
         ;there was an error. use a new RAM table instead...
         jmp cleardifficultytablemem
loaddifficultytableHSCcontinue
         ; parse the data into the HS memory...
         ldy #0
         ldx #0
loaddifficultytableScores
         lda (HSGameTableLo),y
         sta temp1
         iny
         lda (HSGameTableLo),y
         sta temp2
         jsr decodeHSCInitials ; takes 2 byte initials from temp1,2 and stores 3 byte initials in temp3,4,5
         iny
         lda (HSGameTableLo),y
         sta HSRAMScores,x
         lda temp3
         sta HSRAMInitials,x
         inx
         iny
         lda (HSGameTableLo),y
         sta HSRAMScores,x
         lda temp4
         sta HSRAMInitials,x
         inx
         iny
         lda (HSGameTableLo),y
         sta HSRAMScores,x
         lda temp5
         sta HSRAMInitials,x
         inx
         iny
         cpx #15
         bne loaddifficultytableScores
         rts

decodeHSCInitials
         ; takes 2 byte initials from temp1,2 and stores 3 byte initials in temp3,4,5
         ; 2 bytes are packed in the form: 22211111 22_33333
         lda #0
         sta temp4

         lda temp1
         and #%00011111
         sta temp3

         lda temp2
         and #%00011111
         sta temp5

         lda temp1
         asl
         rol temp4
         asl
         rol temp4
         asl
         rol temp4
         lda temp2
         asl
         rol temp4
         asl
         rol temp4
         rts

encodeHSCInitials
         ; takes 3 byte initials from temp3,4,5 and stores 2 byte initials in temp1,2
         ; 2 bytes are packed in the form: 22211111 22_33333
         ; start with packing temp1...
         lda temp4
         and #%00011100
         sta temp1
         asl temp1
         asl temp1
         asl temp1
         lda temp3
         and #%00011111
         ora temp1
         sta temp1
         ; ...temp1 is now packed, on to temp2...
         lda temp5
         asl 
         asl
         ror temp4
         ror
         ror temp4
         ror
         sta temp2
         rts 

findindexHSCerror
         ;the HSC is stuffed. return the bad slot flag
         ldy #$ff
         sty hsgameslot
         rts

findindexHSC
HSCGameID1         = $1029
HSCGameID2         = $106E
HSCGameDifficulty         = $10B3
HSCGameIndex         = $10F8
         ; routine responsible for finding the game index from HSC
         ; call with x=0 to create a new table if none exist, call with x=$ff to avoid creating new tables
         ; the HS loading routine will use x=$ff, the HS saving routine will use x=0
         ldy #69 ; start +1 to account for the dey
findindexHSCloop
         dey
         bmi findindexHSCerror
         lda HSCGameDifficulty,y 
         cmp #$7F
         beq findourindexHSC
         cmp gamedifficulty
         bne findindexHSCloop
         lda HSCGameID1,y 
         cmp #HSIDHI
         bne findindexHSCloop
         lda HSCGameID2,y 
         cmp #HSIDLO
         bne findindexHSCloop
findourindexHSC
         ; if we're here we found our index in the table
         ; or we found the first empty one
         sty hsgameslot
         jsr setupHSCGamepointer ; setup the pointer to the HS Table for this game...
         rts


initializeHSCtableentry
         ldy hsgameslot 
         ; we need to make a new entry...
         lda #HSIDHI
         sta HSCGameID1,y
         lda #HSIDLO
         sta HSCGameID2,y
         lda gamedifficulty
         sta HSCGameDifficulty,y 
         ldx #0
fixHSDGameDifficultylistLoop
         inx
         txa
         sta HSCGameIndex,y
         iny
         cpy #69
         bne fixHSDGameDifficultylistLoop
         rts

setupHSCGamepointer
         ; this routines sets (HSGameTableLo) pointing to the game's HS table
         lda #$17
         sta HSGameTableHi
         lda #$FA
         sta HSGameTableLo
setupHSCGamepointerLoop
         lda HSGameTableLo
         sec
         sbc #25
         sta HSGameTableLo
         lda HSGameTableHi
         sbc #0
         sta HSGameTableHi
         iny
         cpy #69
         bne setupHSCGamepointerLoop
         rts
         
loaddifficultytableAVOX
         ; routine responsible for loading the difficulty table from Avox
         ; we reuse HSC routines to format data to/from our Avox RAM buffer...
         lda #>(eeprombuffer+3)
         sta HSGameTableHi
         lda #<(eeprombuffer+3)
         sta HSGameTableLo

         ; the start location in EEPROM, subtract 32...
         lda #$5F
         sta HSVoxHi
         lda #$E0
         sta HSVoxLo
         lda #0
         sta temp1
loaddifficultytableAVOXloop
         inc temp1
         beq loaddifficultytableAVOXfull
         clc
         lda HSVoxLo
         adc #32
         sta HSVoxLo
         lda HSVoxHi
         adc #0
         sta HSVoxHi
         lda #3
         jsr AVoxReadBytes ; read in 3 bytes, ID1,ID2,Difficulty
         lda eeprombuffer
         cmp #$FF
         beq loaddifficultytableAVOXempty
         cmp #HSIDHI
         bne loaddifficultytableAVOXloop
         lda eeprombuffer+1
         cmp #HSIDLO
         bne loaddifficultytableAVOXloop
         lda eeprombuffer+2
         cmp gamedifficulty
         bne loaddifficultytableAVOXloop
loaddifficultytableAVOXdone
         lda #32
         jsr AVoxReadBytes
         jsr loaddifficultytableHSCcontinue
         rts
loaddifficultytableAVOXfull
         lda #0
         sta hsdevice ; looks like all 255 entries are taken... disable it.
loaddifficultytableAVOXempty
         jmp cleardifficultytablemem         
         rts

cleardifficultytablemem
         ldy #29
         lda #0
cleardifficultytablememloop
         sta HSRAMTable,y
         dey
         bpl cleardifficultytablememloop
         rts
         
     endif ; HSSUPPORT

